import logging

from streaming_data_types import DESERIALISERS
from streaming_data_types.exceptions import StreamingDataTypesException
from streaming_data_types.utils import get_schema

logger = logging.getLogger("saluki")


def _fallback_deserialiser(payload: bytes) -> str:
    return payload.decode()


def try_to_deserialise_message(payload: bytes) -> str:
    logger.debug(f"got some data: {payload}")
    schema = get_schema(payload)
    deserialiser = (
        _fallback_deserialiser  # Fall back to this if we need to so data isn't lost
    )
    try:
        deserialiser = DESERIALISERS[schema]
    except StreamingDataTypesException:
        pass  # TODO
    except KeyError:
        pass
    return deserialiser(payload)



import logging

from confluent_kafka import Consumer, TopicPartition
from saluki import try_to_deserialise_message

logger = logging.getLogger("saluki")


def consume(
    broker: str,
    topic: str,
    partition: int = 0,
    num_messages: int = 1,
    offset: int | None = None,
    go_forwards: bool = False,
) -> None:
    """
    consume from a topic and deserialise each message

    :param broker: the broker address, including the port
    :param topic: the topic to use
    :param partition: the partition to listen to (default is all partitions in a given topic)
    :param num_messages: number of messages to consume
    :param offset: offset to consume from/to
    :param go_forwards: whether to consume forwards or backwards
    :return: None
    """
    c = Consumer(
        {
            "bootstrap.servers": broker,
            "group.id": "saluki",
        }
    )

    if go_forwards:
        if offset is None:
            logger.error("Can't go forwards without an offset")
            return
        start = offset
    else:
        if offset is not None:
            start = offset - num_messages
        else:
            start = (
                c.get_watermark_offsets(TopicPartition(topic, partition), cached=False)[
                    1
                ]
                - num_messages
            )

    logger.info(f"starting at {start}")
    c.assign([TopicPartition(topic, partition, start)])

    try:
        logger.info(f"consuming {num_messages} messages")
        msgs = c.consume(num_messages)
        for msg in msgs:
            if msg is None:
                continue
            if msg.error():
                logger.error("Consumer error: {}".format(msg.error()))
                continue
            if partition is not None and msg.partition() != partition:
                continue
            deserialised = try_to_deserialise_message(msg.value())
            logger.info(f"{msg.offset()}: {deserialised}")
    except Exception as e:
        logger.error(e)
    finally:
        logger.debug(f"closing consumer {c}")
        c.close()



import argparse
import sys


import logging
from logging import FileHandler


from saluki.consume import consume
from saluki.listen import listen

logger = logging.getLogger("saluki")
logging.basicConfig(level=logging.INFO)

_LISTEN = "listen"
_PRODUCE = "produce"
_CONSUME = "consume"


def main():
    parser = argparse.ArgumentParser(
        prog="saluki",
        description="serialise/de-serialise flatbuffers and consume/produce from/to kafka",
    )

    parent_parser = argparse.ArgumentParser(add_help=False)
    parent_parser.add_argument("-b", "--broker", required=True, type=str)
    parent_parser.add_argument("-t", "--topic", required=True, type=str)

    parent_parser.add_argument(
        "-X",
        "--kafka-config",
        help="kafka options to pass through to librdkafka",
        required=False,
        default="",
    )
    parent_parser.add_argument(
        "-l",
        "--log-file",
        help="filename to output all data to",
        required=False,
        default=None,
        type=argparse.FileType("a"),
    )

    sub_parsers = parser.add_subparsers(
        help="sub-command help", required=True, dest="command"
    )

    consumer_parser = argparse.ArgumentParser(add_help=False)
    consumer_parser.add_argument(
        "-e",
        "--entire",
        help="show all elements of an array in a message (truncated by default)",
        default=False,
        required=False,
    )

    consumer_mode_parser = sub_parsers.add_parser(
        _CONSUME, help="consumer mode", parents=[parent_parser, consumer_parser]
    )
    consumer_mode_parser.add_argument(
        "-m",
        "--messages",
        help="How many messages to go back",
        type=int,
        required=False,
        default=1,
    )
    consumer_mode_parser.add_argument(
        "-o", "--offset", help="offset to consume from", type=int, required=False
    )
    consumer_mode_parser.add_argument(
        "-s", "--schema", required=False, default="auto", type=str
    )
    consumer_mode_parser.add_argument(
        "-g", "--go-forwards", required=False, action="store_true"
    )
    consumer_mode_parser.add_argument(
        "-p", "--partition", required=False, type=int, default=0
    )

    listen_parser = sub_parsers.add_parser(
        _LISTEN,
        help="listen mode - listen until KeyboardInterrupt",
        parents=[parent_parser, consumer_parser],
    )
    listen_parser.add_argument(
        "-p", "--partition", required=False, type=int, default=None
    )

    # Producer mode - add this later
    producer_parser = sub_parsers.add_parser(
        _PRODUCE, help="producer mode", parents=[parent_parser]
    )
    producer_parser.add_argument(
        "-f",
        "--filename",
        help="JSON file to produce",
        required=True,
        type=argparse.FileType("r"),
    )

    if len(sys.argv) == 1:
        parser.print_help()
        sys.exit(1)
    args = parser.parse_args()

    if args.log_file:
        logger.addHandler(FileHandler(args.log_file.name))

    if args.command == _LISTEN:
        listen(args.broker, args.topic, args.partition)
    elif args.command == _CONSUME:
        consume(
            args.broker,
            args.topic,
            args.partition,
            args.messages,
            args.offset,
            args.go_forwards,
        )
    elif args.command == _PRODUCE:
        raise NotImplementedError


if __name__ == "__main__":
    main()



import logging

from confluent_kafka import Consumer, TopicPartition
from saluki import try_to_deserialise_message

logger = logging.getLogger("saluki")


def listen(broker: str, topic: str, partition: int | None = None) -> None:
    """
    Listen to a topic and deserialise each message
    :param broker: the broker address, including the port
    :param topic: the topic to use
    :param partition: the partition to listen to (default is all partitions in a given topic)
    :return: None
    """
    c = Consumer(
        {
            "bootstrap.servers": broker,
            "group.id": "saluki",
        }
    )
    c.subscribe([topic])
    if partition is not None:
        c.assign([TopicPartition(topic, partition)])
    try:
        logger.info(f"listening to {broker}/{topic}")
        while True:
            msg = c.poll(1.0)
            if msg is None:
                continue
            if msg.error():
                logger.error("Consumer error: {}".format(msg.error()))
                continue
            if partition is not None and msg.partition() != partition:
                continue
            deserialised = try_to_deserialise_message(msg.value())
            logger.info(f"{msg.offset()}: {deserialised}")
    except KeyboardInterrupt:
        logger.debug("finished listening")
    finally:
        logger.debug(f"closing consumer {c}")
        c.close()
